package bussinesslogic;

import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

import soap.PSCSOAPClient;
import soap.SOAPError;

public class SOPGRefundMerchantClient {

	private HashMap<String, String> transaction = new HashMap<String, String>();
	private SOAPError error = new SOAPError();
	private PSCSOAPClient client;
	private boolean productive;
	private HashMap<String, String> refundResult = new HashMap<String, String>();

	private static final String TEST_URL = "https://soatest.paysafecard.com/psc/services/PscService?wsdl";
	private static final String PRODUCTIVE_URL = "https://soa.paysafecard.com/psc/services/PscService?wsdl";

	public SOPGRefundMerchantClient(String username, String password, boolean productive) {
		transaction.put("username", username);
		transaction.put("password", password);
		this.productive = productive;
	}

	public void setTransaction(String amount, String currency, String mtid, String rtid, String utcOffset) {

		if (rtid != null && !rtid.equals(""))
			transaction.put("rtid", rtid);
		else
			transaction.put("rtid", generateRtid());

		transaction.put("mtid", mtid);

		amount = amount.replace(",", ".");
		DecimalFormat df = new DecimalFormat("#.00");
		df.setRoundingMode(RoundingMode.HALF_UP);
		transaction.put("amount", df.format(Double.parseDouble(amount)));

		transaction.put("currency", currency.toUpperCase());

		if (utcOffset == null || utcOffset.equals(""))
			utcOffset = "+01:00";
		transaction.put("utcOffset", utcOffset);
	}

	public void setCustomer(String merchantClientId, String customerId, String customerIdType) {
		if (customerIdType == null || customerIdType.equals(""))
			customerIdType = "EMAIL";
		transaction.put("customerIdType", customerIdType);

		transaction.put("customerId", customerId);
		transaction.put("merchantClientId", merchantClientId);
	}

	public HashMap<String, String> refund(boolean validationOnly) throws Exception {
		if (validationOnly)
			transaction.put("validationOnly", "true");
		else
			transaction.put("validationOnly", "false");

		if (client == null)
			connect();

		client.addElement("refund", transaction, null);

		if (productive)
			client.doCall(PRODUCTIVE_URL);
		else
			client.doCall(TEST_URL);

		refundResult();

		return refundResult;
	}

	private void refundResult() throws Exception {
		refundResult.put("status", client.getElement("errorCode").equals("0")
				&& client.getElement("resultCode").equals("0") ? "OK" : "ERROR");

		if (client.getElement("errorCode").equals("0") && client.getElement("resultCode").equals("0")) {
			refundResult.put("errorMsg", "");
		} else {
			setError(client.getElement("errorCode"), client.getElement("errorCodeDescription"));
			refundResult.put("errorMsg", error.getErrorMessage());
		}

		refundResult.put("rtid", client.getElement("rtid"));
		refundResult.put("mtid", client.getElement("mtid"));
		refundResult.put("validationOnly", client.getElement("validationOnly"));

		refundResult.put("requestedCurrency", client.getElement("requestedCurrency"));
		refundResult.put("requestedAmount", client.getElement("requestedAmount"));
		refundResult.put("refundedInCurrency", client.getElement("refundedInCurrency"));
		refundResult.put("refundedAmount", client.getElement("refundedAmount"));

		refundResult.put("resultCode", client.getElement("resultCode"));
		refundResult.put("errorCode", client.getElement("errorCode"));
		refundResult.put("errorCodeDescription", client.getElement("errorCodeDescription"));
	}

	private void connect() throws Exception {
		client = new PSCSOAPClient();
	}

	private String generateRtid() {
		Date today = new Date();
		DateFormat format = new SimpleDateFormat("ddMMyyyy");

		return "refundTest_" + format.format(today) + "_" + ((int) (Math.random() * 9999 + 1));
	}

	public HashMap<String, HashMap<String, String>> getDetails() {
		HashMap<String, HashMap<String, String>> details = new HashMap<String, HashMap<String, String>>();
		details.put("transaction", transaction);
		details.put("refundResult", refundResult);

		return details;
	}

	private void setError(String errorCode, String errorCodeDesc) {
		if (errorCodeDesc == null)
			errorCodeDesc = "";

		switch (errorCode) {
		case "3184":
			error.setErrorMessage("Original payment transaction does not exist.");
			break;
		case "3162":
			error.setErrorMessage("E-mail address is not registered with mypaysafecard.");
			break;
		case "3182":
			error.setErrorMessage("Merchantclient-ID is a mandatory parameter.");
			break;
		case "3181":
			error.setErrorMessage("Merchantlcient-ID is not matching with original transaction");
			break;
		default:
			error.setErrorMessage("A technical error occurred. ErrorCode: " + errorCode + " - " + errorCodeDesc
					+ ". Please contact your support department.");
		}
		error.setErrorCode(errorCode);
	}
}
